"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = builder;
exports.configure = exports.Body = exports.Col = exports.Row = exports.Condition = exports.Focus = exports.Tabs = exports.Default = exports.Block = exports.Collapse = exports.Layout = void 0;

var _construct2 = _interopRequireDefault(require("@babel/runtime/helpers/construct"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _models = require("./models");

var _layout = require("./layout");

var _visitorManager = _interopRequireDefault(require("./visitorManager"));

var _validator = _interopRequireDefault(require("./validator"));

var _configure = _interopRequireDefault(require("./configure"));

var _utils = require("./utils");

/** @jsx builder */
// layout
var Layout = 'Layout';
exports.Layout = Layout;

var Collapse = function Collapse(_ref) {
  var attributes = _ref.attributes,
      children = _ref.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "collapse"
  }, attributes), children);
};

exports.Collapse = Collapse;

var Block = function Block(_ref2) {
  var attributes = _ref2.attributes,
      children = _ref2.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "block"
  }, attributes), children);
};

exports.Block = Block;

var Default = function Default(_ref3) {
  var attributes = _ref3.attributes,
      children = _ref3.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "default"
  }, attributes), children);
};

exports.Default = Default;

var Tabs = function Tabs(_ref4) {
  var attributes = _ref4.attributes,
      children = _ref4.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "tabs"
  }, attributes), children);
};

exports.Tabs = Tabs;

var Focus = function Focus(_ref5) {
  var attributes = _ref5.attributes,
      children = _ref5.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "focus"
  }, attributes), children);
};

exports.Focus = Focus;

var Condition = function Condition(_ref6) {
  var attributes = _ref6.attributes,
      children = _ref6.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "condition"
  }, attributes), children);
};

exports.Condition = Condition;

var Row = function Row(_ref7) {
  var attributes = _ref7.attributes,
      children = _ref7.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "row"
  }, attributes), children);
};

exports.Row = Row;

var Col = function Col(_ref8) {
  var attributes = _ref8.attributes,
      children = _ref8.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "col"
  }, attributes), children);
};

exports.Col = Col;

var Body = function Body(_ref9) {
  var attributes = _ref9.attributes,
      children = _ref9.children;
  return builder(Layout, (0, _extends2.default)({
    ui: "body"
  }, attributes), children);
}; // validator config


exports.Body = Body;
var configure = _configure.default.configure; // jsx pragma

exports.configure = configure;

function builder(tag, attributes) {
  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  // flattern children
  children = children.reduce(function (result, child) {
    return result.concat(child);
  }, []).filter(function (child) {
    return child;
  });
  children = (attributes || {}).children || children;

  if (typeof tag === 'function') {
    return tag({
      attributes: attributes,
      children: children
    });
  }

  attributes = attributes || {}; // transform some value to react intl component

  i18n(attributes);

  switch (tag) {
    case 'string':
      return createJSON(_models.StringModel, [attributes, children]);

    case 'number':
      return createJSON(_models.NumberModel, [attributes, children]);

    case 'boolean':
      return createJSON(_models.BooleanModel, [attributes, children]);

    case 'array':
      return createJSON(_models.ArrayModel, [attributes, children]);

    case 'object':
      return createJSON(_models.ObjectModel, [attributes, children]);

    case 'arrayType':
      return createJSON(_models.ArrayModel, [(0, _objectSpread2.default)({}, attributes, {
        defOnly: true
      }), children]);

    case 'objectType':
      return createJSON(_models.ObjectModel, [(0, _objectSpread2.default)({}, attributes, {
        defOnly: true
      }), children]);

    case 'geoPoint':
      return createJSON(_models.GeoPointModel, [attributes, children]);

    case 'dateTime':
      return createJSON(_models.DateModel, [attributes, children]);

    case 'file':
      return createJSON(_models.FileModel, [attributes, children]);

    case 'image':
      return createJSON(_models.ImageModel, [attributes, children]);

    case 'relation':
      return createJSON(_models.RelationModel, [attributes, children]);

    case 'json':
      return createJSON(_models.JsonModel, [attributes, children]);

    case 'component':
      return createJSON(_models.ComponentModel, [attributes, children]);

    case 'enum':
      return new createJSON(_models.EnumModel, [attributes, children]);

    case 'page':
      return new _models.PageModel(attributes, children).toJson();

    case 'root':
      return new _models.RootModel(attributes, children).toJson();

    case 'toolbar':
      {
        return children.reduce(function (result, child) {
          result[child.type] = (0, _objectSpread2.default)({}, child);
          return result;
        }, (0, _objectSpread2.default)({}, attributes, {
          __TOOLBAR__: true
        }));
      }

    case 'sorter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'sorter'
        });
      }

    case 'pagination':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'pagination'
        });
      }

    case 'textFilter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'text'
        });
      }

    case 'numberFilter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'number'
        });
      }

    case 'dateFilter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'date'
        });
      }

    case 'relationSelectFilter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'relation'
        });
      }

    case 'selectFilter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'select'
        });
      }

    case 'filter':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'filter',
          filters: children
        });
      }

    case 'actions':
      {
        return (0, _objectSpread2.default)({}, attributes, children.reduce(function (result, child) {
          result[child.type] = child;
          return result;
        }, {}), {
          type: 'actions'
        });
      }

    case 'export':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'export'
        });
      }

    case 'import':
      {
        return (0, _objectSpread2.default)({}, attributes, {
          type: 'import'
        });
      }

    case 'Layout':
      {
        var insertionVistor = (0, _layout.createLayoutVisitor)(attributes, children).visitor;

        _visitorManager.default.addVisitor(insertionVistor);

        if (attributes.injectValue) {
          var injectVistor = (0, _layout.createLayoutVisitor)((0, _objectSpread2.default)({}, attributes, {
            layoutType: 'injection'
          }), children).visitor;

          _visitorManager.default.addVisitor(injectVistor);
        }

        return children;
      }

    default:
      throw new Error("unsupported type '".concat(tag, "'"));
  }
}

function createJSON(Model, args) {
  var json = (0, _construct2.default)(Model, (0, _toConsumableArray2.default)(args)).toJson();

  _validator.default.validate(json);

  return json;
}

function i18n(attrs) {
  ['title', 'description', 'uiParams', 'label', 'options', 'placeholder', 'fields'].forEach(function (key) {
    if (key in attrs) {
      attrs[key] = (0, _utils.getIntlMessage)(attrs[key]);
    }
  });
}