"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createLayoutVisitor = createLayoutVisitor;
exports.createInsertionLayout = createInsertionLayout;
exports.createInjectionLayout = createInjectionLayout;
exports.getRandomKey = getRandomKey;
exports.CANNER_KEY = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var CANNER_KEY = '__CANNER_KEY__';
exports.CANNER_KEY = CANNER_KEY;

function createLayoutVisitor(attrs, children) {
  var getCannerKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getRandomKey;
  var layoutType = attrs.layoutType;

  switch (layoutType) {
    case 'injection':
      {
        return createInjectionLayout(attrs, children, getCannerKey);
      }

    case 'insertion':
    default:
      {
        return createInsertionLayout(attrs, children, getCannerKey);
      }
  }
}

function createInsertionLayout(attrs, children) {
  var getCannerKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getRandomKey;

  if (!children || !children.length) {
    throw new Error('Layout required one child at least');
  }

  var component = attrs.component,
      ui = attrs.ui,
      title = attrs.title,
      description = attrs.description;

  if (!component && !ui) {
    throw new Error('Layout should have one of properties `component` and `ui` at least.');
  }

  var cannerKey = getCannerKey(); // add cannerKey in the children of layout

  children.forEach(function (child) {
    if (child[CANNER_KEY]) {
      child[CANNER_KEY].push(cannerKey);
    } else {
      child[CANNER_KEY] = [cannerKey];
    }
  });

  var visitor = function visitor(path) {
    var children = path.node.children;
    if (!children) return;

    if (ui === 'body' && CANNER_KEY in path.node && path.node[CANNER_KEY].indexOf(cannerKey) > -1) {
      var _layout = (0, _objectSpread3.default)({}, attrs, {
        nodeType: 'layout.body',
        ui: "body",
        children: [path.node]
      });

      path.node.hideTitle = true;
      path.node.inBody = true;
      path.tree.setNode(path.route, _layout);
      return;
    } // we use cannerKey to find the children should put into the new layout component


    var childrenOfLayout = children.filter(function (child) {
      return child[CANNER_KEY] && child[CANNER_KEY].indexOf(cannerKey) !== -1;
    }); // if there is any children is the layout, the component tree don't have to be changed

    if (!childrenOfLayout.length) {
      return;
    } // create a layout node with its children


    var layout = (0, _objectSpread3.default)({}, attrs, (0, _defineProperty2.default)({
      nodeType: "layout.".concat(attrs.ui),
      childrenName: childrenOfLayout.map(function (child) {
        return child.keyName;
      }),
      title: title,
      description: description,
      children: childrenOfLayout
    }, CANNER_KEY, childrenOfLayout.reduce(function (result, child) {
      return result.concat(child[CANNER_KEY] || []);
    }, [])));
    var meetChildOfLayout = false; // change the children of the node
    // they should become a layout node and others children node that is not included in the layout node.

    var newChildren = children.reduce(function (result, child) {
      // find those excluded children
      if (!child[CANNER_KEY] || child[CANNER_KEY].indexOf(cannerKey) === -1) {
        result.push(child);
      } // add the layout node into children and ignore those children including the layout node
      // meetChildOfLayout is used to replace the first included child node with the layout node


      if (!meetChildOfLayout && child[CANNER_KEY] && child[CANNER_KEY].indexOf(cannerKey) !== -1) {
        meetChildOfLayout = true;
        result.push(layout);
      }

      return result;
    }, []); // update children

    path.tree.setChildren(path.route, newChildren);
  };

  return {
    visitor: {
      'component.object.fieldset': {
        exit: visitor
      },
      'component.array': {
        exit: visitor
      },
      'page.page': {
        exit: visitor
      }
    },
    cannerKey: cannerKey
  };
}

function createInjectionLayout(attrs, children) {
  var getCannerKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getRandomKey;

  if (!children || !children.length) {
    throw new Error('Layout required one child at least');
  }

  var injectValue = attrs.injectValue;

  if (!injectValue) {
    throw new Error('Injection Layout need a injectValue');
  }

  var cannerKey = getCannerKey(); // add cannerKey in the children of layout

  children.forEach(function (child) {
    if (child[CANNER_KEY]) {
      child[CANNER_KEY].push(cannerKey);
    } else {
      child[CANNER_KEY] = [cannerKey];
    }
  });

  var visitor = function visitor(path) {
    if (!path.node[CANNER_KEY] || path.node[CANNER_KEY].indexOf(cannerKey) === -1) {
      return;
    }

    Object.keys(injectValue).forEach(function (key) {
      path.node[key] = injectValue[key];
    });
  };

  var allVisitor = children.reduce(function (result, child) {
    if ("component.".concat(child.type) in result) {
      return result;
    }

    result["component.".concat(child.type)] = visitor;
    return result;
  }, {});
  return {
    visitor: allVisitor,
    cannerKey: cannerKey
  };
}

function getRandomKey() {
  return Math.random().toString(36).substr(2, 10);
}