"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseGraphqlClient = parseGraphqlClient;
exports.parseGraphqlClients = parseGraphqlClients;
exports.parseConnector = parseConnector;
exports.parseConnectors = parseConnectors;
exports.parseResolvers = parseResolvers;
exports.parseSchema = parseSchema;
exports.getIntlMessage = getIntlMessage;
exports.genStorages = genStorages;
exports.componentMap = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _objectPath = _interopRequireDefault(require("object-path"));

var _models = require("./models");

var React = _interopRequireWildcard(require("react"));

var _reactIntl = require("react-intl");

var _lodash = require("lodash");

// this componentMap is the default component in CannerCMS
var componentMap = {
  _map: {
    string: {
      input: '@canner/antd-string-input',
      card: '@canner/antd-string-card',
      editor: '@canner/antd-string-editor',
      link: '@canner/antd-string-link',
      radio: '@canner/antd-string-radio',
      select: '@canner/antd-string-select',
      textarea: '@canner/antd-string-textarea',
      time: '@canner/antd-string-time_picker',
      markdown: '@canner/antd-string-mde',
      default: '@canner/antd-string-input'
    },
    number: {
      input: '@canner/antd-number-input',
      rate: '@canner/antd-number-rate',
      slider: '@canner/antd-number-slider',
      default: '@canner/antd-number-input'
    },
    array: {
      tab: '@canner/antd-array-tabs',
      tabs: '@canner/antd-array-tabs',
      tabTop: '@canner/antd-array-tab_top',
      tabRight: '@canner/antd-array-tab_right',
      tabLeft: '@canner/antd-array-tab_left',
      tabBottom: '@canner/antd-array-tab_bottom',
      gallery: '@canner/antd-array-gallery',
      table: '@canner/antd-array-table',
      slider: '@canner/antd-array-slider',
      panel: '@canner/antd-array-panel',
      tag: '@canner/antd-array-tag',
      tableRoute: '@canner/antd-array-table_route',
      tree: '@canner/antd-array-tree',
      default: '@canner/antd-array-table'
    },
    object: {
      variants: '@canner/antd-object-variants',
      options: '@canner/antd-object-options',
      editor: '@canner/antd-object-editor',
      fieldset: '@canner/antd-object-fieldset',
      default: '@canner/antd-object-fieldset'
    },
    json: {
      variants: '@canner/antd-object-variants',
      options: '@canner/antd-object-options',
      editor: '@canner/antd-object-editor',
      fieldset: '@canner/antd-object-fieldset',
      default: '@canner/antd-object-fieldset'
    },
    relation: {
      singleSelect: '@canner/antd-relation-single_select',
      multipleSelect: '@canner/antd-relation-multiple_select',
      multipleSelectTree: '@canner/antd-relation-multiple_select_tree',
      singleSelectTree: '@canner/antd-relation-single_select_tree',
      default: '@canner/antd-relation-single_select'
    },
    boolean: {
      switch: '@canner/antd-boolean-switch',
      card: '@canner/antd-boolean-card',
      default: '@canner/antd-boolean-switch'
    },
    geoPoint: {
      default: '@canner/antd-object-map'
    },
    file: {
      file: '@canner/antd-object-image',
      default: '@canner/antd-object-image'
    },
    image: {
      image: '@canner/antd-object-image',
      default: '@canner/antd-object-image'
    },
    dateTime: {
      dateTime: '@canner/antd-string-date_time_picker',
      default: '@canner/antd-string-date_time_picker'
    },
    enum: {
      select: '@canner/antd-string-select',
      radio: '@canner/antd-string-radio',
      default: '@canner/antd-string-select'
    },
    component: {
      default: '@canner/antd-string-input'
    }
  },
  get: function get(type) {
    var ui = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

    if (!(type in this._map)) {
      throw new Error("there is no type ".concat(type));
    }

    if (!(ui in this._map[type])) {
      throw new Error("there is no ui ".concat(ui));
    }

    return _objectPath.default.get(this._map, "".concat(type, ".").concat(ui), ui);
  },
  set: function set(path, name) {
    var paths = path.split('.');

    _objectPath.default.set(this._map, paths, name);
  }
};
exports.componentMap = componentMap;

function parseGraphqlClient(attributes, children) {
  var defaultGraphqlClient = attributes.graphqlClient;
  var hasGraphqlClientOnKey = children.find(function (child) {
    return child.graphqlClient;
  });
  return hasGraphqlClientOnKey ? undefined : defaultGraphqlClient;
}

function parseGraphqlClients(attributes, children) {
  var defaultGraphqlClient = attributes.graphqlClient;
  var hasGraphqlClientOnKey = children.find(function (child) {
    return child.graphqlClient;
  });

  if (hasGraphqlClientOnKey) {
    return children.reduce(function (result, child) {
      if (child.graphqlClient) {
        result[child.keyName] = child.graphqlClient;
      } else {
        result[child.keyName] = defaultGraphqlClient;
      }

      return result;
    }, {});
  }

  return undefined;
}

function parseConnector(attributes, children) {
  var defaultConnector = attributes.connector;
  var hasConnectorOnKey = children.find(function (child) {
    return child.connector;
  });
  return hasConnectorOnKey ? undefined : defaultConnector;
}

function parseConnectors(attributes, children) {
  var defaultConnector = attributes.connector;
  var hasConnectorOnKey = children.find(function (child) {
    return child.connector;
  });

  if (hasConnectorOnKey) {
    return children.reduce(function (result, child) {
      if (child.connector) {
        result[child.keyName] = child.connector;
      } else {
        result[child.keyName] = defaultConnector;
      }

      return result;
    }, {});
  }

  return undefined;
}

function parseResolvers(attributes, children) {
  var resolverOnRoot = attributes.resolver;

  if (resolverOnRoot) {
    if ((0, _typeof2.default)(resolverOnRoot) !== 'object') {
      throw new Error('the resolver on root should be a resolver map');
    }

    return resolverOnRoot;
  } else {
    return children.reduce(function (result, child) {
      if (child.resolver) {
        result[child.keyName] = child.resolver;
      }

      return result;
    }, {});
  }
}

function parseSchema(attributes, children) {
  var root = new _models.RootModel(attributes, children);
  return root.toJson();
}

function isI18nFormat(str) {
  return typeof str === 'string' && str.match(/\$\{(.*)\}/);
}

function formatMessage(value) {
  var match = isI18nFormat(value);

  if (match) {
    return React.createElement(_reactIntl.FormattedHTMLMessage, {
      id: match[1]
    });
  }

  return value;
}

function getIntlMessage(value) {
  if ((0, _lodash.isPlainObject)(value)) {
    return (0, _lodash.mapValues)(value, getIntlMessage);
  } else if (Array.isArray(value)) {
    return value.map(function (item) {
      return getIntlMessage(item);
    });
  } else {
    return formatMessage(value);
  }
}

function genStorages(attrs, children) {
  var storageKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'fileStorages';
  return children.reduce(function (result, child) {
    result[child.keyName] = child[storageKey] || attrs[storageKey] || {};
    return result;
  }, {});
}