"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _utils = require("./utils");

var TYPES = ['string', 'number', 'boolean', 'array', 'object', 'relation', 'json', 'component'];
var CANNER_DATA_TYPES = Object.keys(_utils.componentMap._map);

var Validator = function Validator() {
  var _this = this;

  (0, _classCallCheck2.default)(this, Validator);
  (0, _defineProperty2.default)(this, "throwError", true);
  (0, _defineProperty2.default)(this, "setConfig", function (config) {
    if ('throwError' in config) {
      _this.throwError = config.throwError;
    }
  });
  (0, _defineProperty2.default)(this, "validatePackageName", function (packageName) {
    var type = _this.parseType(packageName);

    if (TYPES.indexOf(type) === -1) {
      var message = "Invalid package name '".concat(packageName, "', type should be one of ").concat(JSON.stringify(TYPES));
      return _this.invalid(message);
    }

    return true;
  });
  (0, _defineProperty2.default)(this, "validateCannerDataType", function (cannerDataType) {
    if (CANNER_DATA_TYPES.indexOf(cannerDataType) === -1) {
      var message = "Invalid canner data type '".concat(cannerDataType, ", should be one of ").concat(JSON.stringify(CANNER_DATA_TYPES), "'");
      return _this.invalid(message);
    }

    return true;
  });
  (0, _defineProperty2.default)(this, "validate", function (json) {
    var packageName = json.packageName,
        type = json.type;

    if (type === 'component') {
      return true;
    }

    if (packageName === '@canner/antd-object-fieldset') {
      return true;
    }

    return _this.validatePackageName(packageName) && _this.validateCannerDataType(type);
  });
  (0, _defineProperty2.default)(this, "invalid", function () {
    var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    if (_this.throwError) {
      throw new Error(message);
    } else {
      // eslint-disable-next-line
      console.error(message);
      return false;
    }
  });
  (0, _defineProperty2.default)(this, "parseType", function (packageName) {
    return packageName.replace(/\\/g, '/').split('/').slice(-1)[0].split('-')[1];
  });
};

var _default = new Validator();

exports.default = _default;